@@ -1,10 +1,19 @@
 	/**
 	 * Facilities to manipulate 64-bit numbers encoded as strings.
-	 * The JavaScript implementation of openradtool uses strings for
-	 * numbers because of the 64-bit problem: internally,
-	 * openradtool uses 64-bit integer numbers, but JavaScript has
-	 * only 53 bits of precision for integers.
-	 * This class is a modified version of long.js fixed to base 10.
+	 * The JavaScript implementation of openradtool uses strings for numbers
+	 * because of the 64-bit problem: internally, openradtool uses 64-bit
+	 * integer numbers, but JavaScript has only 53 bits of precision for
+	 * integers.
+	 * This class is a modified version of long.js fixed to base 10 and
+	 * converted to TypeScript.
+	 *
+	 * Original source: https://github.com/dcodeIO/long.js
+	 *
+	 * @license
+	 * Copyright 2009 The Closure Library Authors.
+	 * Copyright 2020 Daniel Wirtz / The long.js Contributors.
+	 * Copyright 2021 Kristaps Dzonsons
+	 * SPDX-License-Identifier: Apache-2.0
 	 */
 	export class Long {
 		private readonly __isLong__: boolean = true;
@@ -24,14 +33,11 @@
 				false: unsigned;
 		}
 
-		static readonly ZERO: Long = 
-			new Long(0, 0);
-		static readonly ONE: Long = 
-			new Long(1, 0);
-		static readonly UZERO: Long = 
-			new Long(0, 0, true);
-		static readonly TEN_TO_EIGHT: Long =
-			new Long(100000000, 0);
+		static readonly ZERO: Long = new Long(0, 0);
+		static readonly ONE: Long = new Long(1, 0);
+		static readonly UZERO: Long = new Long(0, 0, true);
+		static readonly TEN_TO_EIGHT: Long = new Long(100000000, 0);
+
 		static readonly MIN_VALUE: Long = 
 			new Long(0, 0x80000000|0, false);
 		static readonly MAX_UNSIGNED_VALUE: Long = 
@@ -92,7 +98,8 @@
 		}
 
 		/**
-		 * @return Whether the value is strictly < 0.
+		 * @return Whether the value is strictly < 0.  This only applies
+		 * to signed numbers, else this is always false.
 		 */
 		isNegative(): boolean
 		{
@@ -100,7 +107,7 @@
 		}
 
 		/**
-		 * @return Whether the value is zero.
+		 * @return Whether the value is zero, regardless of sign.
 		 */
 		isZero(): boolean
 		{
@@ -116,7 +123,11 @@
 		}
 
 		/**
-		 * @return Whether the value equals the given value.
+		 * @return Whether the value equals the given value.  This is
+		 * according to the numerical value, not the bit pattern, so a
+		 * negative signed value and a positive signed value will be
+		 * the same even if having the same bit pattern.  The exception
+		 * here is zero, which is the same whether signed or not.
 		 */
 		eq(other: Long|number): boolean
 		{
@@ -152,30 +163,72 @@
 
 		/**
 		 * @return The bit-wise AND of the value with the given value.
-		 * If the given value can't be parsed, returns zero.
+		 * Sign is inherited from the source value.  
 		 */
-		and(other: Long|number|string|null): Long
+		and(other: Long|number): Long
 		{
-			const v: Long|null = !Long.isLong(other) ?
-				Long.fromValue(other) : <Long>other;
-
-			if (v === null)
-				return Long.ZERO;
-
+			const v: Long = !Long.isLong(other) ?
+				Long.fromNumber(<number>other) : <Long>other;
 			return new Long(this.low & v.low, 
 					this.high & v.high, 
 					this.unsigned);
 		}
 
 		/**
-		 * @return The value left-shifted by the given number of
-		 * bits.
+		 * @return The bit-wise OR of the value with the given value.
+		 * Sign is inherited from the source value.  If the
+		 * parameterised value can't be parsed, returns zero.
+		 */
+		or(other: Long|number): Long 
+		{
+			const v: Long = !Long.isLong(other) ?
+				Long.fromNumber(<number>other) : <Long>other;
+			return new Long(this.low | v.low, 
+					this.high | v.high, 
+					this.unsigned);
+		}
+
+		/**
+		 * @return 0 if they are the same (or the number could not be
+		 * parsed), 1 if the this is greater, and -1 if the given one
+		 * is greater.
+		 */
+		compare(other: Long|number): number
+		{
+			const v: Long = !Long.isLong(other) ?
+				Long.fromNumber(<number>other) : <Long>other;
+
+			if (this.eq(v))
+				return 0;
+
+			const thisNeg: boolean = this.isNegative();
+			const otherNeg: boolean = v.isNegative();
+
+			if (thisNeg && !otherNeg)
+				return -1;
+			if (!thisNeg && otherNeg)
+				return 1;
+
+			// At this point the sign bits are the same
+
+			if (!this.unsigned)
+				return this.sub(v).isNegative() ? -1 : 1;
+
+			// Both are positive if at least one is unsigned
+
+			return (v.high >>> 0) > (this.high >>> 0) || 
+				(v.high === this.high && 
+				 (v.low >>> 0) > (this.low >>> 0)) ?  -1 : 1;
+		};
+
+		/**
+		 * @return The value left-shifted by the given number of bits.
+		 * Sign is inherited from the source value.
 		 */
 		shl(numBits: Long|number): Long
 		{
 			let v: number = Long.isLong(numBits) ?
-				(<Long>numBits).toInt() : 
-				<number>numBits;
+				(<Long>numBits).toInt() : <number>numBits;
 
 			if ((v &= 63) === 0)
 				return this;
@@ -197,8 +250,7 @@
 		mul(tomul: Long|number): Long
 		{
 			const v: Long = !Long.isLong(tomul) ?
-				Long.fromNumber(<number>tomul) : 
-				<Long>tomul;
+				Long.fromNumber(<number>tomul) : <Long>tomul;
 
 			if (this.isZero() || v.isZero())
 				return Long.ZERO;
@@ -262,6 +314,16 @@
 				 this.unsigned);
 		}
 
+		/**
+		 * @return The difference of the value and the given value.
+		 */
+		sub(tosub: Long|number): Long
+		{
+			const v: Long = !Long.isLong(tosub) ?
+				Long.fromNumber(<number>tosub) : <Long>tosub;
+			return this.add(v.neg());
+		};
+
 		/**
 		 * @return The sum of the value and the given value.
 		 */
@@ -416,6 +478,9 @@
 				return nresult.neg();
 			}
 
+			if (!str.match(/^[0-9]+$/))
+				return null;
+
 			// Do several (8) digits each time through the
 			// loop, so as to  minimize the calls to the
 			// very expensive emulated div.
@@ -425,6 +490,8 @@
 					Math.min(8, str.length - i);
 				const value: number = 
 					parseInt(str.substring(i, i + size), 10);
+				if (isNaN(value))
+					return null;
 				if (size < 8) {
 					const power: Long = 
 						Long.fromNumber
